define('aim/absence/view/NewFilterPopupView',['aim/util/Utils',
	        'aim/absence/view/AvailableResourcesView',
			'aim/absence/view/FilteredResourcesView',
			'aim/absence/view/AddResourcesButtonView',
			'aim/absence/view/FilterPopupHeaderView',
			'aim/absence/view/FilterPopupFooterView'], function(
					Utils,
		AvailableResourcesView,
		FilteredResourcesView,
		AddResourcesButtonView,
		FilterPopupHeaderView,
		FilterPopupFooterView
		) {
	var NewFilterPopupView = Backbone.View.extend({
		el: '#new-resource-filter-dialog',

		events: {
			"click #new-filter-cancel-button": "cancel",
			"click #new-filter-apply-button": "apply",
			"click #new-filter-save-button": "save"
		},
		
		initialize: function() {
			var newFilterModel = this.model.get("newFilterModel");
			this.listenTo(newFilterModel, 'sync', this.render);
			var availableResourcesView = new AvailableResourcesView({model: newFilterModel});
			var filteredResourcesView = new FilteredResourcesView({model: newFilterModel});
			var addResourcesButtonView = new AddResourcesButtonView({model: newFilterModel});
			var filterPopupHeaderView = new FilterPopupHeaderView({model: newFilterModel});
			return this;
		},
			
		render: function() {
			if(!this.filterPopupFooterView){
				this.filterPopupFooterView = new FilterPopupFooterView({model: this.model.get("newFilterModel")});
			}
			return this;
		},
		
		cancel: function(event) {
			AJS.dialog2("#new-resource-filter-dialog").hide();
		},
		
		apply: function(event) {
			var message = this.model.get("newFilterModel").attributes.errorMessage;
			if( message )return this;
			this.model.get("newFilterModel").set("operation","applyFilter");
			var that = this;
			this.model.get("newFilterModel").save(null, {success:function(model, result, options) {
				var valid = Utils.validateBarModel(model, 'naomi.templates.js.jirarp.board.message', '#filter-dialog-message-bar');
				if(valid){
					AJS.dialog2("#new-resource-filter-dialog").hide();
					that.model.get("filterContextModel").set("selectedFilter",result.filterId);
					that.model.get("filterContextModel").save();
					that.model.get("contextModel").set("selectedFilter",result.filterId);
					that.model.get("contextModel").set("resourceFilterChanged","true");
					that.model.get("contextModel").save();
					AJS.$("#error-field-filtername").text("");
					AJS.$("#findProjects").val("");
					AJS.$("#checkBoxProjectFilter").attr('checked', false);
					AJS.$("#findResources").val("");
					AJS.$("#checkBoxResourceFilter").attr('checked', false);
					return this;
				}
			},error:function(model, res, options) {
				var title = 'Error while applying changes';
				var message = 'Error while applying changes';
				Utils.sendBarErrorMessage('#filter-dialog-message-bar', title, message, false, 5000, 500);
				Utils.markFieldWithError(selector, fieldHolder);
			}});
		},
		
		save: function(event) {
			var message = this.model.get("newFilterModel").attributes.errorMessage;
			if( message )return this;
			if( this.model.get("newFilterModel").attributes.filterName ){
				this.model.get("newFilterModel").set("operation","changeFilter");	
			}else{
				if( ! this.model.get("newFilterModel").attributes.changedFilterName){//no name given
					var title = AJS.I18n.getText("naomi.templates.js.jirarp.filter.dialog.message.error.no.filter.name");	
					Utils.sendBarErrorMessage('#filter-dialog-message-bar', title, title, false, 5000, 500);
					return false;
				}
				this.model.get("newFilterModel").set("operation","saveFilter");
				this.model.get("newFilterModel").attributes.filterName = this.model.get("newFilterModel").attributes.changedFilterName;
			}
			var that = this;
			this.model.get("newFilterModel").save(null, {success:function(model, result, options) {
				var valid = Utils.validateBarModel(model, 'naomi.templates.js.jirarp.board.message', '#filter-dialog-message-bar');
				if(valid){
					AJS.dialog2("#new-resource-filter-dialog").hide();
					that.model.attributes.filterContextModel.attributes.selectedFilter = result.filterId;
					that.model.attributes.filterContextModel.save();
					that.model.attributes.contextModel.attributes.selectedFilter = result.filterId;
					that.model.attributes.contextModel.attributes.operation = 'filterChanged';
					that.model.attributes.contextModel.save();
					AJS.$("#error-field-filtername").text("");
					AJS.$("#findProjects").val("");
					AJS.$("#checkBoxProjectFilter").attr('checked', false);
					AJS.$("#findResources").val("");
					AJS.$("#checkBoxResourceFilter").attr('checked', false);
					return this;
				}
			}});	
		}
	});
	return NewFilterPopupView;
});