define('aim/assignment/view/BoardMainView',['aim/util/Utils',
                                            'aim/assignment/model/ChangePlanValueModel',
                                            'aim/assignment/model/PlanValueModel',
                                            'aim/assignment/model/AssignmentBoardModeModel',
                                            'aim/analytics/model/AnalyticsBoardViewModeModel',
                                            'aim/main/view/SingleSelectView',
                                            'aim/main/view/SplitButtonView',
                                            'aim/assignment/view/SelectCalendarWeekView',
                                            'aim/assignment/view/SelectProjectResourceActionView',
                                            'aim/analytics/view/AnalyticsResourceView',
                                            'aim/analytics/view/AnalyticsProjectView',
                                            'aim/assignment/view/PlanValueView'], function(
                                            		Utils,
                                            		ChangePlanValueModel,
                                            		PlanValueModel,
                                            		AssignmentBoardModeModel,
                                            		AnalyticsBoardViewModeModel,
                                            		SingleSelectView,
                                            		SplitButtonView,
                                            		SelectCalendarWeekView,
                                            		SelectProjectRessourceActionView,
                                            		AnalyticsResourceView,
                                            		AnalyticsProjectView,
                                            		PlanValueView) {
	var BoardMainView = Backbone.View.extend({
		el : "#jira",
		
		events: {
			"click #resourceContextButton" : "toggleResources",
			"click #projectContextButton" : "toggleProjects",
			"click #resourceOnlyContextButton" : "toggleOnlyResources",
			"click #analyticsProjectButton" : "toggleAnalyticsProject",
			"click #analyticsResourceButton" : "toggleAnalyticsResource",
			"click #analyticsResourceOnlyButton" : "toggleAnalyticsResourceOnly",
			"click #dialog-add-plan-button" : "addPlanValues",
			"click #calendar-weeks-shift-left-button" : "calendarWeeksShiftLeft",
			"click #calendar-weeks-shift-right-button" : "calendarWeeksShiftRight",
			"click .select-board-mode-options" : "selectBoardMode",
			"click .select-analytics-view-mode-options" : "selectAnalyticsViewMode",
			"change .inputweeks" : "edit",
			"change #calendar-week-select" : "jumpToCalendarWeek"
		},
		
		context : {},
		
		initialize : function() {
			this.listenTo(this.model, 'sync', this.render);
			this.assignmentBoardModeModel = new AssignmentBoardModeModel();
			var selectBoardMode = new SplitButtonView({el: BoardMainView.selectBoardModeEl, model: this.assignmentBoardModeModel});
			this.assignmentBoardModeModel.fetch();
			this.analyticsBoardViewModeModel = new AnalyticsBoardViewModeModel();
			var selectAnalyticsViewMode = new SplitButtonView({el: BoardMainView.selectAnalyticsViewModeEl, model: this.analyticsBoardViewModeModel});
			this.analyticsBoardViewModeModel.fetch();
			this.planValueModel = new PlanValueModel();

			var requests = new Array();
			AJS.$.ajaxSetup({
				beforeSend:function(event,xhr,options){
					requests.push(xhr);
					if(xhr.url.endsWith("/context") ){
						AJS.$('.spinner').spin();
						AJS.$('.spinner').addClass('naomi-spinner');
					}
				},
				complete:function(xhr,status){
					requests.pop(xhr);
					if(requests.length == 0){
						AJS.$('.spinner').spinStop();
						AJS.$('.spinner').removeClass('naomi-spinner');
					}
				}
			});	
			this.context = {};
			this.context.planMode = {};
			this.context.anaylticsMode = {};
			this.context.planMode = constants.projectMode;
			this.context.analyticsMode = constants.analyticsProjectMode;
			var selectCalendarWeekView = new SelectCalendarWeekView({model: this.model});
			this.model.fetch();
			this.fieldsInError = new Array();
		},
		
		render : function() {
			if(this.assignmentBoardModeModel.get('selectedValue') == '1'){
				this.fetchPlanView();
			}else if(this.assignmentBoardModeModel.get('selectedValue') == '2'){
				this.renderAnalyticsView();
			}
		    if(! this.selectProjectRessourceActionView)
		    	this.selectProjectRessourceActionView = new SelectProjectRessourceActionView({model: this.model}); 
		    if( ! this.model.attributes.errorMessage && ! this.model.attributes.warnMessage && ! this.model.attributes.infoMessage && this.fieldsInError.length == 0){
				Utils.clearFieldMarker(this.fieldsInError);
			}
			return this;
		},
		
		calendarWeeksShiftLeft : function() {
			this.model.set("calendarWeekShift",-1);
			this.model.set("operation","shiftCalendarWeek");
			this.model.save();
			this.options.weekDataModel.set("calendarWeekShift",this.options.weekDataModel.attributes.calendarWeekShift -1);
		},
		
		calendarWeeksShiftRight : function() {
			this.model.set("calendarWeekShift",1);
			this.model.set("operation","shiftCalendarWeek");
			this.model.save();
			this.options.weekDataModel.set("calendarWeekShift",this.options.weekDataModel.attributes.calendarWeekShift +1);
		},
		
		jumpToCalendarWeek : function(event) {
			var value = event.currentTarget.value;
			this.model.set("calendarWeekShift",value);
			this.model.set("operation","shiftCalendarWeek");
			this.model.save();
			this.options.weekDataModel.set("calendarWeekShift",this.options.weekDataModel.attributes.calendarWeekShift + parseInt(event.currentTarget.value));
		},
		
		selectBoardMode : function(event){
			var selectedMode = event.currentTarget.dataset.value;
			this.assignmentBoardModeModel.set('selectedValue',selectedMode);
			this.assignmentBoardModeModel.save();
			this.render();
		},
		
		selectAnalyticsViewMode: function(event){
			var selectedMode;
			if(event){
				selectedMode = event.currentTarget.dataset.value;
				this.analyticsBoardViewModeModel.set('selectedValue',selectedMode);
				this.analyticsBoardViewModeModel.save();
			}else{
				selectedMode = this.analyticsBoardViewModeModel.get('selectedValue');
			}
			if(selectedMode == 1){//difference
				this.disableViews('.analytics-plan');
				this.disableViews('.analytics-log');
			}else if(selectedMode == 2){//difference + plan
				this.enableViews('.analytics-plan');
				this.disableViews('.analytics-log');
			}else if(selectedMode == 3){//difference + log
				this.disableViews('.analytics-plan');
				this.enableViews('.analytics-log');
			}else if(selectedMode == 4){//all
				this.enableViews('.analytics-plan');
				this.enableViews('.analytics-log');
			}
		},
		
		enableViews: function(clazz){
			AJS.$(clazz).removeClass('naomi-invisible');
		},
		
		disableViews: function(clazz){
			AJS.$(clazz).addClass('naomi-invisible');
		},
		
		
		toggleOnlyResources : function() {
			if(this.context.planMode == constants.resourceOnly) {
				return;
			}
			this.context.planMode = constants.resourceOnly;
			this.$('#projectButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#resourceButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#resourceOnlyButtonLi').addClass( 'aui-nav-selected' );
			this.renderPlanView(this.model, this.options.weekDataModel.attributes);
		},
		
		toggleResources : function() {
			if(this.context.planMode == constants.resourceMode) {
				return;
			}
			this.context.planMode = constants.resourceMode;
			this.$('#projectButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#resourceOnlyButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#resourceButtonLi').addClass( 'aui-nav-selected' );
			this.renderPlanView(this.model, this.options.weekDataModel.attributes);
		},
		
		toggleProjects : function() {
			if(this.context.planMode == constants.projectMode) {
				return;
			}			
			this.context.planMode = constants.projectMode;
			this.$('#resourceButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#resourceOnlyButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#projectButtonLi').addClass( 'aui-nav-selected' );
			this.renderPlanView(this.model, this.options.weekDataModel.attributes);
		},
		
		toggleAnalyticsProject : function() {
			if(this.context.analyticsMode == constants.analyticsProjectMode) {
				return;
			}			
			this.context.analyticsMode = constants.analyticsProjectMode;
			this.$('#analyticsProjectButtonLi').addClass( 'aui-nav-selected' );
			this.$('#analyticsResourceButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#analyticsResourceOnlyButtonLi').removeClass( 'aui-nav-selected' );
			this.render();
		},
		
		toggleAnalyticsResource : function() {
			if(this.context.analyticsMode == constants.analyticsResourceMode) {
				return;
			}			
			this.context.analyticsMode = constants.analyticsResourceMode;
			this.$('#analyticsProjectButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#analyticsResourceButtonLi').addClass( 'aui-nav-selected' );
			this.$('#analyticsResourceOnlyButtonLi').removeClass( 'aui-nav-selected' );
			this.render();
		},

		toggleAnalyticsResourceOnly : function() {
			if(this.context.analyticsMode == constants.analyticsResourceOnlyMode) {
				return;
			}			
			this.context.analyticsMode = constants.analyticsResourceOnlyMode;
			this.$('#analyticsProjectButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#analyticsResourceButtonLi').removeClass( 'aui-nav-selected' );
			this.$('#analyticsResourceOnlyButtonLi').addClass( 'aui-nav-selected' );
			this.render();
		},

		addPlanValues : function() {
			this.planValueModel.set('operation','initializeDialogValues');
			this.planValueModel.set('currentCalendarWeek', this.model.attributes.calendarWeekViewStart);
			if( ! this.planValueView)
				this.planValueView = new PlanValueView({model: this.planValueModel, contextModel: this.model});
			this.planValueModel.save();
		    AJS.dialog2("#add-plan-value-dialog").show();
		},
		
		fetchPlanView : function() {
			this.$('#resourceButtonLi').removeClass( 'naomi-invisible' );
			this.$('#resourceOnlyButtonLi').removeClass( 'naomi-invisible' );
			this.$('#projectButtonLi').removeClass( 'naomi-invisible' );
			this.$('#dialog-add-plan-button').removeClass('naomi-invisible');
			this.$('#analyticsProjectButtonLi').addClass('naomi-invisible');
			this.$('#analyticsResourceButtonLi').addClass('naomi-invisible');
			this.$('#analyticsResourceOnlyButtonLi').addClass('naomi-invisible');
			this.$('#select-analytics-view-mode-li').addClass('naomi-invisible');
			var self = this;
			if( ! this.options.weekDataModel.get('weeks')){
				this.options.weekDataModel.fetch({
					success: function (model, response, options) {
				    	self.renderPlanView(model,response);
			    },
			     error: function () {
			    	//TODO message
			    }});
			}else{
				this.options.weekDataModel.save(null, {
					 success: function (model, response, options) {
					    	self.renderPlanView(model,response);
				    },
				     error: function () {
				    	//TODO message
				    }});
			}
		},
		
		renderPlanView: function(model,response) {
			focusedElementId = AJS.$(':focus').attr('id');
			var tmpOps = {};
			tmpOps.mode = this.context.planMode;
			tmpOps.currentWeek = this.model.get('highlightedWeek');
			tmpOps.projects = this.model.get('filteredProjects');
			tmpOps.resources = this.model.get('filteredResources');
			tmpOps.weeks = response.weeks;
			var pagePanelInner = this.$('#aui-page-panel-inner');
			pagePanelInner.html(Naomi.Templates.JiraRp.board(tmpOps));
			if(focusedElementId){
			    AJS.$('#' + focusedElementId).focus();
			    AJS.$('#' + focusedElementId).select();
			}
		},
		
		renderWithoutFetch: function() {
			focusedElementId = AJS.$(':focus').attr('id');
			var tmpOps = {};
			tmpOps.mode = this.context.planMode;
			tmpOps.currentWeek = this.model.get('highlightedWeek');
			tmpOps.projects = this.model.get('filteredProjects');
			tmpOps.resources = this.model.get('filteredResources');
			tmpOps.weeks = this.options.weekDataModel.attributes.weeks;
			var pagePanelInner = this.$('#aui-page-panel-inner');
			pagePanelInner.html(Naomi.Templates.JiraRp.board(tmpOps));
			if(focusedElementId){
			    AJS.$('#' + focusedElementId).focus();
			    AJS.$('#' + focusedElementId).select();
			}
		},
		
		renderAnalyticsView : function() {
			this.$('#resourceButtonLi').addClass( 'naomi-invisible' );
			this.$('#resourceOnlyButtonLi').addClass( 'naomi-invisible' );
			this.$('#projectButtonLi').addClass( 'naomi-invisible' );
			this.$('#analyticsProjectButtonLi').removeClass('naomi-invisible');
			this.$('#analyticsResourceButtonLi').removeClass('naomi-invisible');
			this.$('#analyticsResourceOnlyButtonLi').removeClass('naomi-invisible');
			this.$('#select-analytics-view-mode-li').removeClass('naomi-invisible');
			this.$('#dialog-add-plan-button').addClass('naomi-invisible');
			var tmpOps = {};
			tmpOps.analyticsMode = this.context.analyticsMode;
			tmpOps.projects = this.model.get('filteredProjects');
			tmpOps.resources = this.model.get('filteredResources');
			tmpOps.weeks = this.model.get('weeks');
			tmpOps.currentWeek = this.model.get('highlightedWeek');
			var pagePanelInner = this.$('#aui-page-panel-inner');
			pagePanelInner.html(Naomi.Templates.JiraRp.analyticsBoard(tmpOps, null, this.model.get('context')));
			if(this.context.analyticsMode == constants.analyticsResourceMode || this.context.analyticsMode == constants.analyticsResourceOnlyMode){
		    	if(!this.analyticsResourceView)
		    		this.analyticsResourceView = new AnalyticsResourceView({model: this.model});
		    	this.analyticsResourceView.render();
		    }
		    if(this.context.analyticsMode == constants.analyticsProjectMode){
		    	if(!this.analyticsProjectView)
		    		this.analyticsProjectView = new AnalyticsProjectView({model: this.model});
		    	this.analyticsProjectView.render();
		    }
		    this.selectAnalyticsViewMode();
		},
		
		edit: function(event) {
			var that = this;
			var longResourceId = event.currentTarget.dataset.longresourceid;
			var resourceId = event.currentTarget.dataset.resourceid;
			var longProjectId = event.currentTarget.dataset.longprojectid;
			var projectId = event.currentTarget.dataset.projectid;
			var weekId = event.currentTarget.dataset.weekid;
			var weeks = this.model.attributes.weeks;
			var year = event.currentTarget.dataset.year;
			focusedElementId = AJS.$(':focus').attr('id');
			var selector = '#'+event.currentTarget.id;
			var fieldHolder = this.fieldsInError;

			var modelForSave = new ChangePlanValueModel();
			modelForSave.set('selectedProject', longProjectId);
			modelForSave.set('selectedResource', longResourceId);
			modelForSave.set('year', year);
			modelForSave.set('week', weekId);
			modelForSave.set('planValue', event.currentTarget.value);
			
			modelForSave.save(null, {success:function(model, res, options) {
				if( Utils.validateFlagModel(model,'naomi.templates.js.jirarp.board.message', selector, fieldHolder) ){
					var context = that.options.weekDataModel.attributes;
					var result = res;
					var kw = _.find(context.weeks, function(week) {
						if(week.title == weekId) {
							return true;
						} else {
							return false;
						}
					}); 
					
					var restSum =result.resourceRestSum;
					var sum = result.resourceSum;
					var inputValue = result.planValue;
					var projectSum = result.projectSum;
					var effectiveAssignmentLevel=result.effectiveAssignmentLevel;
	
					kw.projectSubSections[projectId].sectionValues[resourceId] = inputValue;
					kw.projectSubSections[projectId].sectionValues[resourceId].effectiveAssignmentLevel=effectiveAssignmentLevel;
					kw.projectSubSections[projectId].overallSum = projectSum;
					
					kw.resourceSubSections[resourceId].sectionValues[projectId] = inputValue;
					kw.resourceSubSections[resourceId].overallSum = sum;
					kw.resourceSubSections[resourceId].sectionValues[projectId].effectiveAssignmentLevel=effectiveAssignmentLevel;
					kw.resourceSubSections[resourceId].restSum=restSum;
					kw.resourceSubSections[resourceId].effectiveAssignmentLevel=effectiveAssignmentLevel;
					
					_.each(kw.projectSubSections, function(resourcePlaningProject) {
						if(resourcePlaningProject.sectionValues[resourceId]){
							resourcePlaningProject.sectionValues[resourceId].effectiveAssignmentLevel=effectiveAssignmentLevel;
						}
					});
					that.renderWithoutFetch();
					if( res.errorMessage || res.warnMessage || res.infoMessage ){
						//in this case the validate model has sent a message, keep quite
					}else{
						var title= AJS.I18n.getText("naomi.templates.js.jirarp.plan.board.message.saved.value.title");
						var replacements = {'%VALUE%': inputValue+'','%USER%': model.attributes.changedResource.displayName, '%PROJECT%': model.attributes.changedProject.name, '%WEEK%': weekId};
						var text = AJS.I18n.getText("naomi.templates.js.jirarp.plan.board.message.saved.value.message");
						var message = Utils.replace(replacements,text);
						Utils.sendMessage('success',title,'auto',message);
						Utils.unmarkField(selector, fieldHolder);
					}
				}
				
			},error:function(model, res, options) {
				var title= AJS.I18n.getText("naomi.templates.js.jirarp.plan.board.message.error.value.title");
				var text = AJS.I18n.getText("naomi.templates.js.jirarp.plan.board.message.error.value.message");
				var responseMessage = jQuery.parseJSON(res.responseText);
				var replacements = {'%VALUE%': event.currentTarget.value+'','%RESPONSE_MESSAGE%' : responseMessage.message};
				var message = Utils.replace(replacements,text);
				Utils.sendMessage('error',title,'manual',message);
				Utils.markFieldWithError(selector, fieldHolder);
			}});				
			
			if(focusedElementId){
				AJS.$('#' + focusedElementId).focus();
				AJS.$('#' + focusedElementId).select();
			}
		}
	},{//constants
		selectBoardModeEl     		: '#select-board-mode',
		selectAnalyticsViewModeEl 	: '#select-analytics-view-mode'
	});
	return BoardMainView;
});