define('aim/capacity/view/BoardMainView',['aim/util/Utils',
                                          'aim/capacity/model/CapacityValueModel',
                                          'aim/capacity/view/SelectCalendarWeekView',
                                          'aim/capacity/view/CapacityValueView'], function(Utils,CapacityValueModel,SelectCalendarWeekView,CapacityValueView) {
	var BoardMainView = Backbone.View.extend({
		el : "#content",
		
		events: {
			"click #dialog-add-capacity-button" : "addCapacityValues",
			"click #calendar-weeks-shift-left-button" : "calendarWeeksShiftLeft",
			"click #calendar-weeks-shift-right-button" : "calendarWeeksShiftRight",
			"change .inputweeks" : "edit",
			"change #calendar-week-select" : "jumpToCalendarWeek"
		},
		
		context : {},
		
		initialize : function() {
			var requests = new Array();
			AJS.$.ajaxSetup({
				beforeSend:function(event,xhr,options){
					requests.push(xhr);
					if(xhr.url.endsWith("/context") ){
						AJS.$('.spinner').spin();
						AJS.$('.spinner').addClass('naomi-spinner');
					}
				},
				complete:function(xhr,status){
					requests.pop(xhr);
					if(requests.length == 0){
						AJS.$('.spinner').spinStop();
						AJS.$('.spinner').removeClass('naomi-spinner');
					}
				}
			});	
			this.listenTo( this.model, 'sync', this.render);
			this.capacityValueModel = new CapacityValueModel();
			var selectCalendarWeekView = new SelectCalendarWeekView({model: this.model});
			AJS.$('#add-capacity-popup').replaceWith(Naomi.Templates.CapacityBoard.addCapacityPopup());
			this.model.fetch();
			this.fieldsInError = new Array();
		},
		
		render : function() {
			focusedElementId = AJS.$(':focus').attr('id');
			var tmpOps = {};
			tmpOps.isResourceMode = this.context.mode == constants.resourceMode ? true : false;	
			tmpOps.currentWeek = this.model.get('highlightedWeek');
			var pagePanelInner = this.$('#aui-page-panel-inner');
			pagePanelInner.html(Naomi.Templates.CapacityBoard.board(tmpOps, null, this.model.get('context')));
		    AJS.$('#' + focusedElementId).focus();
		    AJS.$('#' + focusedElementId).select();
		    if( ! this.model.attributes.errorMessage && ! this.model.attributes.warnMessage && ! this.model.attributes.infoMessage && this.fieldsInError.length == 0){
				Utils.clearFieldMarker(this.fieldsInError);
			}
			return this;
		},
			
		addCapacityValues : function() {
			this.capacityValueModel.set('operation','initializeDialogValues');
			this.capacityValueModel.set('currentCalendarWeek', this.model.attributes.currentCalendarWeek);
			if( ! this.capacityValueView)
				this.capacityValueView = new CapacityValueView({model: this.capacityValueModel, contextModel: this.model});
			this.capacityValueModel.save();
		    AJS.dialog2("#add-capacity-dialog").show();
		},
		
		calendarWeeksShiftLeft : function() {
			this.model.set("calendarWeekShift",-1);
			this.model.set("operation","shiftCalendarWeek");
			this.model.save();
		},
		
		calendarWeeksShiftRight : function() {
			this.model.set("calendarWeekShift",1);
			this.model.set("operation","shiftCalendarWeek");
			this.model.save();
		},
		
		jumpToCalendarWeek : function(event) {
			this.model.set("calendarWeekShift",event.currentTarget.value);
			this.model.set("operation","shiftCalendarWeek");
			this.model.save();
		},
		
		edit: function(event) {
			var that = this;
			if(event && event.currentTarget && event.currentTarget.id) {
				var longResourceId = event.currentTarget.dataset.longresourceid;
				var resourceId = event.currentTarget.dataset.resourceid;
				var weekId = event.currentTarget.dataset.weekid;
				var weeks = this.model.attributes.context.weeks;
				var year = event.currentTarget.dataset.year;
				var fieldHolder = this.fieldsInError;
				var selector = '#'+event.currentTarget.id;

				var modelForSave = new CapacityValueModel();
				modelForSave.set('selectedResource', longResourceId);
				modelForSave.set('week', weekId);
				modelForSave.set('year', year);
				modelForSave.set('operation', "spreadInWeek");
				modelForSave.set('saved', 'true');
				modelForSave.set('capacityValue', event.currentTarget.value);
				modelForSave.save(null, {success:function(model, res, options) {
					var context = that.model.attributes.context;
					var message = model.attributes.message;
					if( Utils.validateFlagModel( model, 'naomi.templates.js.jirarp.board.message', selector, fieldHolder) ){
						var result = res;
						var kw = _.find(context.weeks, function(week) {
							if(week.title == weekId) {
								return true;
							} else {
								return false;
							}
						});
						var restSum =result.singleValueResult;
						kw.resourceSubSections[resourceId].sum = restSum;
						that.render();
						if( res.errorMessage || res.warnMessage || res.infoMessage ){
							//in this case the validate model has sent a message, keep quite
						}else{
							var title= AJS.I18n.getText("naomi.templates.js.jirarp.capacity.board.message.saved.value.title");
							var replacements = {'%VALUE%': restSum+'','%USER%': model.attributes.changedResource.displayName, '%WEEK%': weekId};
							var text = AJS.I18n.getText("naomi.templates.js.jirarp.capacity.board.message.saved.value.message");
							var message = Utils.replace(replacements,text);
							Utils.sendMessage('success',title,'auto',message);
							Utils.unmarkField(selector, fieldHolder);
						}
					}
					
				},error:function(model, res, options) {
					
					var title= AJS.I18n.getText("naomi.templates.js.jirarp.capacity.board.message.error.value.title");
					var text = AJS.I18n.getText("naomi.templates.js.jirarp.capacity.board.message.error.value.message");
					var responseMessage = jQuery.parseJSON(res.responseText);
					var replacements = {'%VALUE%': event.currentTarget.value+'','%RESPONSE_MESSAGE%' : responseMessage.message};
					var message = Utils.replace(replacements,text);
					Utils.sendMessage('error',title,'manual',message);
					Utils.markFieldWithError(selector, fieldHolder);
				}});				
			}
		}
	});
	return BoardMainView;
});